﻿
Imports Microsoft.DirectX
Imports Microsoft.DirectX.Direct3D
Imports System.Runtime.InteropServices


Module Video

    Friend AV1 As AviStream_API
    Friend AV2 As AviStream_API
    Friend DX As VideoDX

End Module



Class AviStream_API

    Private m_FileName As String = ""

    ' ---------------------------------------------------------------
    '  Version 1.7 - removed the DestImage
    ' ---------------------------------------------------------------
    'Private m_DestImage As Image
    'Private m_DestGraphics As Graphics

    Private m_VideoFramesCount As Int32
    Private m_VideoFrameCurrent As Int32
    Private m_VideoTimeSeconds As Single

    Private sw As Diagnostics.Stopwatch = New Diagnostics.Stopwatch
    Private m_ppavi As Integer
    Private m_pGetFrame As Integer
    Private m_pFrameData As Integer
    Private m_asInfo As AviStream_API.AVI_STREAM_INFO
    Private m_bInfo As BITMAPINFO

    Private m_BitCount As Int32
    Friend m_Texture As Texture


    ' ------------------------------------------------------------------------- LOAD VIDEO FILE
    Friend Sub LoadVideoFile(ByVal name As String)

        CloseAll()

        Try
            m_FileName = name

            ' --------------------------------------------------------------------------- OPEN VIDEO SECTION OF THE FILE
            Dim ret As Int32
            ret = AVIStreamOpenFromFile(m_ppavi, m_FileName, FourCC("vids"), 0, OF_SHARE_DENY_WRITE, 0)
            If ret <> 0 Then
                m_ppavi = 0
                Exit Sub
            End If

            ' --------------------------------------------------------------------------- STREAM INFO
            Dim retval As Int32 = AVIStreamInfo(m_ppavi, m_asInfo, Marshal.SizeOf(m_asInfo))
            m_VideoFramesCount = m_asInfo.dwLength
            m_VideoTimeSeconds = CSng(m_VideoFramesCount * m_asInfo.dwScale / m_asInfo.dwRate)

            ' --------------------------------------------------------------------------- GET FRAME
            m_pGetFrame = AVIStreamGetFrameOpen(m_ppavi, AVIGETFRAMEF_BESTDISPLAYFMT)

            If m_pGetFrame = 0 Then
                Exit Sub
            End If

            AVIStreamBeginStreaming(m_ppavi, 0, m_VideoFramesCount, 1000)

            m_pFrameData = AVIStreamGetFrame(m_pGetFrame, 0)

            m_bInfo.bmiHeader = CType(Runtime.InteropServices.Marshal.PtrToStructure(CType(m_pFrameData, IntPtr), _
                                                                                    m_bInfo.bmiHeader.GetType()),  _
                                                                                    BITMAPINFOHEADER)

            m_VideoFrameCurrent = 0
        Catch
            MsgBox("Can not load the file """ & m_FileName & """" & vbCrLf, MsgBoxStyle.Exclamation)
        End Try

    End Sub

    Friend Sub CloseAll()
        If m_ppavi <> 0 Then
            AVIStreamEndStreaming(m_ppavi)
        End If
        If m_pGetFrame <> 0 Then
            AVIStreamGetFrameClose(m_pGetFrame)
            m_pGetFrame = 0
        End If
        If m_ppavi <> 0 Then
            AVIStreamRelease(m_ppavi)
            m_ppavi = 0
        End If
        m_pFrameData = 0
    End Sub

    Friend Function SetVideoPosition(ByVal NormPos As Single) As Boolean
        Dim desiredFrame As Int32 = CInt(NormPos * m_VideoFramesCount)
        If desiredFrame > m_VideoFramesCount - 1 Then desiredFrame = m_VideoFramesCount - 1
        If desiredFrame < 0 Then desiredFrame = 0
        If desiredFrame <> m_VideoFrameCurrent Then
            m_VideoFrameCurrent = desiredFrame
            Return True
        Else
            Return False
        End If
    End Function

    'Private Sub InitDestImage(ByVal DestCtrl As Control)
    '    With DestCtrl
    '        If .Width > 0 And .Height > 0 Then
    '            If m_DestImage Is Nothing OrElse m_DestImage.Width <> .Width OrElse m_DestImage.Height <> .Height Then
    '                m_DestImage = New Bitmap(.Width, .Height)
    '                m_DestGraphics = Graphics.FromImage(m_DestImage)
    '            End If
    '        End If
    '    End With
    'End Sub

    Friend Sub ShowVideoFrame_WithDirect3D(ByVal DestCtrl As Control)
        If DestCtrl Is Nothing Then Return
        If m_ppavi = 0 Or m_pGetFrame = 0 Or m_pFrameData = 0 Then Return

        'InitDestImage(DestCtrl)

        m_pFrameData = AVIStreamGetFrame(m_pGetFrame, m_VideoFrameCurrent)

        If m_pFrameData = 0 Then Exit Sub

        CopyBitmapToTexture()
    End Sub


    Private Sub CopyBitmapToTexture()
        If DX.m_Device Is Nothing Then Return

        Dim SourcePointer As Int32 = m_pFrameData + Marshal.SizeOf(m_bInfo.bmiHeader)
        Dim BmpWidth As Int32 = m_bInfo.bmiHeader.biWidth
        Dim BmpHeight As Int32 = m_bInfo.bmiHeader.biHeight
        Dim BmpBitCount As Int32 = m_bInfo.bmiHeader.biBitCount
        Dim BmpSizeImage As Int32 = m_bInfo.bmiHeader.biSizeImage

        Dim desc As SurfaceDescription
        Dim dw As Int32
        Dim dh As Int32
        Dim destBytes As Int32
        Dim destPointer As Int32
        Dim sourceLineLenght As Int32

        If m_Texture Is Nothing OrElse _
           BmpWidth <> m_Texture.GetLevelDescription(0).Width OrElse _
           BmpHeight <> m_Texture.GetLevelDescription(0).Height OrElse _
           BmpBitCount <> m_BitCount Then

            m_BitCount = BmpBitCount

            Dim TextureFormat As Format = DX.m_PresentParams.BackBufferFormat

            If TextureFormat <> Format.R5G6B5 And BmpBitCount = 16 Then
                TextureFormat = Format.A1R5G5B5
            End If

            Select Case TextureFormat
                Case Format.X8R8G8B8 : TextureFormat = Format.A8R8G8B8
            End Select

            m_Texture = New Texture(DX.m_Device, _
                                    BmpWidth, _
                                    BmpHeight, _
                                    1, _
                                    Usage.SoftwareProcessing, _
                                    TextureFormat, _
                                    Pool.Managed)

        End If
        If BmpSizeImage = 0 Then Exit Sub

        desc = m_Texture.GetLevelDescription(0)
        dw = desc.Width
        dh = desc.Height

        Dim destPitch As Int32

        Dim gs As Microsoft.DirectX.GraphicsStream = m_Texture.LockRectangle(0, LockFlags.None, destPitch) 'lock+unlock = 8mS(640x480) 2.5mS(320x240)

        destPointer = gs.InternalData.ToInt32

        destBytes = dh * destPitch

        Dim nLines As Int32 = BmpHeight
        sourceLineLenght = CInt(BmpSizeImage / nLines)

        If BmpSizeImage = destBytes Then
            ' ------------------------------------------------------------------ normal
            SourcePointer = SourcePointer + (nLines - 1) * sourceLineLenght
            For Y As Int32 = 1 To nLines
                CopyMemory(destPointer, SourcePointer, sourceLineLenght)

                SourcePointer = SourcePointer - sourceLineLenght
                destPointer = destPointer + destPitch
            Next
            ' ------------------------------------------------------------------ reverse
            'CopyMemory(destPointer, SourcePointer, BmpSizeImage)
        End If

        m_Texture.UnlockRectangle(0)

    End Sub

End Class



