﻿
Imports System.Math

Module Module_SaveLoad

    Friend EventsAreEnabled As Boolean = False

    ' =======================================================================================================
    '   APP TITLE AND VERSION
    ' =======================================================================================================
    Friend Function AppTitleAndVersion(Optional ByVal Title As String = "") As String
        If Title = "" Then Title = Replace(My.Application.Info.AssemblyName, "_", " ")
        Dim s() As String = Split(My.Application.Info.Version.ToString, ".")
        Return Title & " - V" & s(0) & "." & s(1)
    End Function


    ' =======================================================================================================
    '   Platform selection
    ' =======================================================================================================
    Friend Function CanUseWindowsDlls() As Boolean
        Select Case Environment.OSVersion.Platform
            Case PlatformID.Win32NT, PlatformID.Win32S, PlatformID.Win32Windows, PlatformID.WinCE
                Return True
            Case Else
                Return False
        End Select
    End Function
    Friend Function PlatformAdjustedFileName(ByVal FileName As String, _
                                             Optional ByVal DefaultName As String = "") As String
        If CanUseWindowsDlls() Then
            FileName = Replace(FileName, "/", "\")
            Return FileName
        Else
            If FileName.Contains(":") Then FileName = DefaultName
            FileName = Replace(FileName, "\", "/")
            Return FileName
        End If
    End Function


    ' =======================================================================================
    '  FORM
    ' =======================================================================================
    Friend Sub LimitFormPosition(ByVal f As System.Windows.Forms.Form)
        If f.WindowState <> FormWindowState.Normal Then Return
        GetMaxScreenBounds()
        EnsureFormVisible(f)
    End Sub

    Private sw As Integer
    Private sh As Integer
    Private Sub GetMaxScreenBounds()
        ' -------------------------------------- Gets an array of all the screens connected to the system.
        Dim Screens() As System.Windows.Forms.Screen = System.Windows.Forms.Screen.AllScreens
        ' -------------------------------------- For each screen
        Dim Index As Integer
        For Index = 0 To Screens.GetUpperBound(0)
            sw = Math.Max(sw, Screens(Index).Bounds.Left + Screens(Index).Bounds.Width)
            sh = Math.Max(sh, Screens(Index).Bounds.Top + Screens(Index).Bounds.Height)
        Next
    End Sub

    'Private Sub EnsureFormCompletelyVisible(ByVal frm As Form)
    '    With frm
    '        .Width = Math.Min(.Width, sw)           ' non oltre le dimensioni di una maximized window
    '        .Height = Math.Min(.Height, sh)         ' non oltre le dimensioni di una maximized window
    '        .Width = Math.Max(.Width, 320)          ' non meno di 320x240
    '        .Height = Math.Max(.Height, 240)        ' non meno di 320x240
    '        .Left = Math.Min(.Left, sw - .Width)    ' non oltre il bordo destro
    '        .Top = Math.Min(.Top, sh - .Height)     ' non oltre il bordo inferiore
    '        .Left = Math.Max(.Left, 0)              ' non meno del bordo sinistro
    '        .Top = Math.Max(.Top, 0)                ' non sopra al bordo superiore
    '    End With
    'End Sub

    Private Sub EnsureFormVisible(ByVal frm As Form)
        With frm
            .Width = Math.Min(.Width, sw)               ' not more than the VIRTUALSCREEN dimensions
            .Height = Math.Min(.Height, sh)             ' not more than the VIRTUALSCREEN dimensions 
            .Width = Math.Max(.Width, 32)               ' at least 32x24
            .Height = Math.Max(.Height, 24)             ' at least 32x24
            .Left = Math.Min(.Left, sw - 50)            ' not beyond the right border - 50 pixels
            .Top = Math.Min(.Top, sh - 50)              ' not beyond the bottom border - 50 pixels
            .Left = Math.Max(.Left, 50 - .Width)        ' at least at the left border + 50 pixels
            .Top = Math.Max(.Top, -30)                  ' at least at the top border
        End With
    End Sub

    ' (The value of the RestoreBounds property is valid only 
    '   when the WindowState property of the Form class is not equal to Normal)
    Friend Function GetFormRectangle(ByVal frm As Form) As Rectangle
        Dim r As Rectangle
        If frm.WindowState = FormWindowState.Normal Then
            r = frm.Bounds
        Else
            r = frm.RestoreBounds
        End If
        Return r
    End Function



    ' ================================================================================================
    '  Private Read-Write functions
    ' ================================================================================================

    Private Function TabString(ByVal Name As String, _
                              Optional ByVal Value As Double = Double.NaN, _
                              Optional ByVal fmt As String = "") As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)
        If Double.IsNaN(Value) Then
            Return Name
        Else
            Return Name & "=" & Strings.StrDup(nTab, " ") & Value.ToString(fmt)
        End If
    End Function

    Private Function TabString(ByVal Name As String, _
                                  ByVal Value As Boolean) As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)

        Return Name & "=" & Strings.StrDup(nTab, " ") & Value.ToString
    End Function

    Private Function TabString(ByVal Name As String, _
                                ByVal Value As String) As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)

        Return Name & "=" & Strings.StrDup(nTab, " ") & Value
    End Function

    Private Function Val_Single(ByVal l As String) As Single
        Return CSng(Val(l.Replace(",", ".")))
    End Function

    Private Function Val_Double(ByVal l As String) As Double
        Return Val(l.Replace(",", "."))
    End Function

    Private Function Val_Int(ByVal l As String) As Int32
        Return CInt(Val(l))
    End Function


    Private Function ExtractParamName(ByRef s As String) As String
        ' ------------------------- Returns the first field from begin to the first "=" symbol
        ' -------------------------  and removes it from the string
        Dim i As Integer
        i = InStr(s, "=")
        If i > 0 Then
            ExtractParamName = Trim(Strings.Left(s, i - 1))
            s = Trim(Mid(s, i + 1))
        Else
            ExtractParamName = Trim(s)
            s = ""
        End If
    End Function

    Private Function AssemblyName() As String
        Return System.Reflection.Assembly.GetExecutingAssembly.GetName.Name
    End Function



    ' ==================================================================================================
    '  SAVE LOAD -- Program INI
    ' ==================================================================================================

    Friend Sub Save_INI()
        Try
            '
            Dim iniFileName As String = Application.StartupPath & "\" & AssemblyName() & "_INI.txt"
            Dim f As System.IO.StreamWriter
            f = IO.File.CreateText(iniFileName)
            '
            f.WriteLine(" Program Params")
            f.WriteLine("===========================================")
            ' ------------------------------------------------------------------------------ FORM BOUNDS
            Dim r As Rectangle
            r = GetFormRectangle(Form1)
            f.WriteLine(TabString("Form1_Top", r.Top))
            f.WriteLine(TabString("Form1_Left", r.Left))
            f.WriteLine(TabString("Form1_Width", r.Width))
            f.WriteLine(TabString("Form1_Height", r.Height))
            f.WriteLine(TabString("VideoMode", Form1.VideoMode))
            f.WriteLine(TabString("NormalLocationX", Form1.NormalLocation.X))
            f.WriteLine(TabString("NormalLocationY", Form1.NormalLocation.Y))
            f.WriteLine(TabString("LargeLocationX", Form1.LargeLocation.X))
            f.WriteLine(TabString("LargeLocationY", Form1.LargeLocation.Y))
            f.WriteLine(TabString("NormalSizeW", Form1.NormalSize.Width))
            f.WriteLine(TabString("NormalSizeH", Form1.NormalSize.Height))
            f.WriteLine(TabString("LargeSizeW", Form1.LargeSize.Width))
            f.WriteLine(TabString("LargeSizeH", Form1.LargeSize.Height))
            'f.WriteLine(TabString("Form1_WindowState", Form1.WindowState))
            '
            f.WriteLine("")
            f.WriteLine(" InOut")
            f.WriteLine("===========================================")
            f.WriteLine(TabString("Enable1", Form1.chk_Enable1.Checked))
            f.WriteLine(TabString("SlotIn1", Form1.txt_SlotIn1.Text))
            f.WriteLine(TabString("SlotChange1", Form1.txt_SlotChange1.Text))
            f.WriteLine(TabString("SlotRed1", Form1.txt_SlotRed1.Text))
            f.WriteLine(TabString("SlotGreen1", Form1.txt_SlotGreen1.Text))
            f.WriteLine(TabString("SlotBlue1", Form1.txt_SlotBlue1.Text))
            f.WriteLine(TabString("Position1", Form1.tk_Position1.Value.ToString))
            '
            f.WriteLine(TabString("Enable2", Form1.chk_Enable2.Checked))
            f.WriteLine(TabString("SlotIn2", Form1.txt_SlotIn2.Text))
            f.WriteLine(TabString("SlotChange2", Form1.txt_SlotChange2.Text))
            f.WriteLine(TabString("SlotRed2", Form1.txt_SlotRed2.Text))
            f.WriteLine(TabString("SlotGreen2", Form1.txt_SlotGreen2.Text))
            f.WriteLine(TabString("SlotBlue2", Form1.txt_SlotBlue2.Text))
            f.WriteLine(TabString("Position2", Form1.tk_Position2.Value.ToString))
            '
            f.WriteLine("")
            f.WriteLine(" Video file")
            f.WriteLine("===========================================")
            f.WriteLine(TabString("VideoFileIndex", Form1.VideoFile1_Index.ToString))
            '
            'f.WriteLine("")
            'f.WriteLine(" examples")
            'f.WriteLine("===========================================")
            'f.WriteLine(TabString("DefaultCapMaxDist", Form1.txt_MaxDist.NumericValue))
            'f.WriteLine(TabString("FastScroll", Form2.btn_FastScroll.Checked))
            'f.WriteLine(TabString("ShowRawCount", Form2.btn_ShowRawCount.Checked))
            'f.WriteLine(TabString("ShowXY", Form2.btn_ShowXY.Checked))
            '
            f.Close()
        Catch
        End Try
    End Sub



    Friend Sub Load_INI()
        ' 
        Dim l As String
        Dim iniFileName As String = Application.StartupPath & "\" & AssemblyName() & "_INI.txt"
        If My.Computer.FileSystem.FileExists(iniFileName) Then
            Dim f As System.IO.StreamReader
            f = IO.File.OpenText(iniFileName)

            Do While Not f.EndOfStream
                l = f.ReadLine()
                Select Case ExtractParamName(l)
                    Case "Form1_Top" : Form1.Top = CInt(Val(l))
                    Case "Form1_Left" : Form1.Left = CInt(Val(l))
                    Case "Form1_Width" : Form1.Width = CInt(Val(l))
                    Case "Form1_Height" : Form1.Height = CInt(Val(l))
                    Case "VideoMode" : Form1.VideoMode = CType(CInt(Val(l)), Form1.VideoModes)
                    Case "NormalLocationX" : Form1.NormalLocation.X = CInt(Val(l))
                    Case "NormalLocationY" : Form1.NormalLocation.Y = CInt(Val(l))
                    Case "LargeLocationX" : Form1.LargeLocation.X = CInt(Val(l))
                    Case "LargeLocationY" : Form1.LargeLocation.Y = CInt(Val(l))
                    Case "NormalSizeW" : Form1.NormalSize.Width = CInt(Val(l))
                    Case "NormalSizeH" : Form1.NormalSize.Height = CInt(Val(l))
                    Case "LargeSizeW" : Form1.LargeSize.Width = CInt(Val(l))
                    Case "LargeSizeH" : Form1.LargeSize.Height = CInt(Val(l))
                        ' -------- restore WindowState is not applicable with this video program
                        'Case "Form1_WindowState" : Form1.WindowState = CType((Val(l)), FormWindowState)
                        ' ----------------------------------------------------------------- InOut
                    Case "Enable1" : Form1.chk_Enable1.Checked = l = "True"
                    Case "SlotIn1" : Form1.txt_SlotIn1.Text = l
                    Case "SlotChange1" : Form1.txt_SlotChange1.Text = l
                    Case "SlotRed1" : Form1.txt_SlotRed1.Text = l
                    Case "SlotGreen1" : Form1.txt_SlotGreen1.Text = l
                    Case "SlotBlue1" : Form1.txt_SlotBlue1.Text = l
                    Case "Position1" : Form1.tk_Position1.Value = CInt(Val(l))
                        '
                    Case "Enable2" : Form1.chk_Enable2.Checked = l = "True"
                    Case "SlotIn2" : Form1.txt_SlotIn2.Text = l
                    Case "SlotChange2" : Form1.txt_SlotChange2.Text = l
                    Case "SlotRed2" : Form1.txt_SlotRed2.Text = l
                    Case "SlotGreen2" : Form1.txt_SlotGreen2.Text = l
                    Case "SlotBlue2" : Form1.txt_SlotBlue2.Text = l
                    Case "Position2" : Form1.tk_Position2.Value = CInt(Val(l))
                        ' ----------------------------------------------------------------- Video file
                    Case "VideoFileIndex" : Form1.VideoFile1_Index = CInt(Val(l))
                        ' ----------------------------------------------------------------- examples
                        'Case "DefaultCapMaxDist" : Form1.txt_MaxDist.NumericValue = Val(l)
                        'Case "FastScroll" : Form2.btn_FastScroll.Checked = l = "True"
                        'Case "ShowRawCount" : Form2.btn_ShowRawCount.Checked = l = "True"
                        'Case "ShowXY" : Form2.btn_ShowXY.Checked = l = "True"
                End Select
            Loop
            f.Close()
        End If
        '
        LimitFormPosition(Form1)
        '
    End Sub




End Module
